<?php

namespace Drupal\better_json_response\EventSubscriber;

use Drupal\better_json_response\BetterJsonResponse;
use Drupal\better_json_response\Form\BetterJsonResponseConfigForm;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\jsonapi\EventSubscriber\ResourceResponseSubscriber;
use Drupal\jsonapi\ResourceResponse;
use Symfony\Component\HttpFoundation\JsonResponse as JsonResponseAlias;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Response subscriber that serializes and removes ResourceResponses data.
 */
class SubResourceResponseSubscriber extends ResourceResponseSubscriber {

  /**
   * Better Json Response config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $betterJsonResponseConfig;

  /**
   * {@inheritDoc}
   */
  public function __construct(SerializerInterface $serializer, ConfigFactoryInterface $configFactory) {
    parent::__construct($serializer);
    $this->betterJsonResponseConfig = $configFactory->get(BetterJsonResponseConfigForm::CONFIG_NAME);
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents(): array {
    // Run after ResourceResponseSubscriber event.
    $events[KernelEvents::RESPONSE][] = ['onResponse', 129];
    return $events;
  }

  /**
   * {@inheritDoc}
   */
  public function onResponse(ResponseEvent $event): void {
    $response = $event->getResponse();
    if (!$response instanceof BetterJsonResponse) {
      return;
    }

    $isCacheActivate = $response->isCacheabilityDefined() && !($this->betterJsonResponseConfig->get('deactivate_cache') ?? FALSE);
    $cacheableMetadata = $response->getCacheableMetadata();
    $originalData = $response->getOriginalData();
    if (!is_array($originalData)) {
      return;
    }
    $request = $event->getRequest();
    // We transform ResourceResponse in json api object.
    $this->transformToJsonApi($originalData, $cacheableMetadata, $request);

    $class = !$isCacheActivate ? JsonResponseAlias::class : CacheableJsonResponse::class;
    $response = new $class($originalData, $response->getStatusCode(), $response->headers->all());
    if ($response instanceof CacheableJsonResponse) {
      $response->addCacheableDependency($cacheableMetadata);
      $extraCacheableMetadata = new CacheableMetadata();
      $extraCacheableMetadata->addCacheTags(['config:' . BetterJsonResponseConfigForm::CONFIG_NAME]);
      $response->addCacheableDependency($extraCacheableMetadata);
    }
    $event->setResponse($response);
  }

  /**
   * Transform $originalData's ResourceResponse object into json api object.
   *
   * @param array $originalData
   *   Array to processed.
   * @param \Drupal\Core\Cache\CacheableMetadata $cacheableMetadata
   *   Cacheable metadata object to ascend child metadata.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Current request.
   */
  protected function transformToJsonApi(array &$originalData, CacheableMetadata &$cacheableMetadata, Request $request): void {
    foreach ($originalData as &$item) {
      if (is_array($item)) {
        $this->transformToJsonApi($item, $cacheableMetadata, $request);
      }
      if (!$item instanceof ResourceResponse) {
        continue;
      }
      $this->renderResponseBody($request, $item, $this->serializer, 'api_json');
      $result = $this->flattenResponse($item, $request);
      $cacheableMetadata->addCacheableDependency($result->getCacheableMetadata());
      $item = Json::decode($result->getContent());
    }
  }

}
